"""MCP SSH Server implementation."""

import json
import logging
import os
import sys
from typing import Dict, List, Optional, Any, Iterator

import paramiko
from pydantic import BaseModel, Field
from mcp import types
from mcp.server import FastMCP

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

class SSHSession:
    """SSH Session that connects to a remote server."""
    
    def __init__(self, hostname: str, port: int, username: str, key_filename: str):
        """Initialize SSH session.
        
        Args:
            hostname: Remote server hostname
            port: SSH port
            username: SSH username
            key_filename: Path to SSH key file
        """
        self.hostname = hostname
        self.port = port
        self.username = username
        self.key_filename = key_filename
        self.client = paramiko.SSHClient()
        self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.connected = False
        
    def connect(self) -> bool:
        """Connect to SSH server.
        
        Returns:
            True if connection successful, False otherwise
        """
        try:
            self.client.connect(
                hostname=self.hostname,
                port=self.port,
                username=self.username,
                key_filename=self.key_filename
            )
            self.connected = True
            return True
        except Exception as e:
            logger.error(f"SSH connection error: {e}")
            return False
            
    def execute_command(self, command: str) -> Dict[str, Any]:
        """Execute a command on the remote server.
        
        Args:
            command: Command to execute
            
        Returns:
            Dictionary with stdout, stderr, and exit_code
        """
        if not self.connected:
            if not self.connect():
                return {"stdout": "", "stderr": "Not connected to SSH server", "exit_code": -1}
        
        try:
            stdin, stdout, stderr = self.client.exec_command(command)
            exit_code = stdout.channel.recv_exit_status()
            
            return {
                "stdout": stdout.read().decode("utf-8"),
                "stderr": stderr.read().decode("utf-8"),
                "exit_code": exit_code
            }
        except Exception as e:
            logger.error(f"Command execution error: {e}")
            return {"stdout": "", "stderr": str(e), "exit_code": -1}
    
    def close(self) -> None:
        """Close SSH connection."""
        if self.connected:
            self.client.close()
            self.connected = False


class SSHConnectionParams(BaseModel):
    """Parameters for SSH connection."""
    
    hostname: Optional[str] = Field(None, description="SSH server hostname or IP address")
    port: Optional[int] = Field(None, description="SSH server port")
    username: Optional[str] = Field(None, description="SSH username")
    key_filename: Optional[str] = Field(None, description="Path to SSH private key file")


class CommandParams(BaseModel):
    """Parameters for executing a command."""
    
    command: str = Field(..., description="Command to execute on remote server")


class SSHServerMCP(FastMCP):
    """MCP server that provides SSH access to remote systems."""
    
    def __init__(self, hostname=None, port=None, username=None, key_filename=None, server_name=None, tool_prefix=None):
        """Initialize SSH server.
        
        Args:
            hostname: SSH server hostname from environment or config
            port: SSH server port from environment or config
            username: SSH username from environment or config
            key_filename: Path to SSH key file from environment or config
            server_name: Custom name for the server instance (for multiple servers)
            tool_prefix: Prefix for tool names (e.g., 'server1_' for 'server1_ssh_connect')
        """
        # Get server name from environment variable or parameter
        server_name = server_name or os.environ.get("MCP_SSH_SERVER_NAME", "SSH Server")
        super().__init__(name=server_name)
        self.ssh_session: Optional[SSHSession] = None
        
        # Get configuration from environment variables or passed parameters
        self.default_hostname = hostname or os.environ.get("MCP_SSH_HOSTNAME")
        self.default_port = port or int(os.environ.get("MCP_SSH_PORT", 22))
        self.default_username = username or os.environ.get("MCP_SSH_USERNAME")
        self.default_key_filename = key_filename or os.environ.get("MCP_SSH_KEY_FILENAME")
        
        # Get tool prefix from parameter or environment variable
        self.tool_prefix = tool_prefix or os.environ.get("MCP_SSH_TOOL_PREFIX", "")
        
        # Register tools with optional prefix
        connect_name = f"{self.tool_prefix}ssh_connect"
        execute_name = f"{self.tool_prefix}ssh_execute"
        disconnect_name = f"{self.tool_prefix}ssh_disconnect"
        
        self.add_tool(self.ssh_connect, name=connect_name, description=f"Connect to SSH server: {server_name}")
        self.add_tool(self.ssh_execute, name=execute_name, description=f"Execute a command on SSH server: {server_name}")
        self.add_tool(self.ssh_disconnect, name=disconnect_name, description=f"Disconnect from SSH server: {server_name}")
    
    def ssh_connect(self, params: SSHConnectionParams) -> Dict[str, Any]:
        """Connect to an SSH server.
        
        Args:
            params: SSH connection parameters (ignored for security-critical fields)
            
        Returns:
            Result of connection attempt
        """
        # Always use config/environment variables for security-critical fields
        hostname = self.default_hostname
        username = self.default_username
        key_filename = self.default_key_filename
        
        # Only allow port to be optionally specified in the request
        port = self.default_port if self.default_port else params.port
        
        # Validate that we have all required parameters
        if not all([hostname, username, key_filename]):
            missing = []
            if not hostname: missing.append("hostname")
            if not username: missing.append("username")
            if not key_filename: missing.append("key_filename")
            return {"success": False, "message": f"Missing required parameters: {', '.join(missing)}"}
        
        self.ssh_session = SSHSession(
            hostname=hostname,
            port=port,
            username=username,
            key_filename=key_filename
        )
        
        if self.ssh_session.connect():
            return {"success": True, "message": f"Connected to {hostname}"}
        else:
            return {"success": False, "message": "Failed to connect to SSH server"}
    
    def ssh_execute(self, params: CommandParams) -> Dict[str, Any]:
        """Execute a command on the SSH server.
        
        Args:
            params: Command parameters
            
        Returns:
            Command execution result
        """
        if not self.ssh_session or not self.ssh_session.connected:
            return {"success": False, "message": "Not connected to SSH server"}
        
        return self.ssh_session.execute_command(params.command)
    
    def ssh_disconnect(self) -> Dict[str, Any]:
        """Disconnect from the SSH server.
        
        Returns:
            Disconnect result
        """
        if not self.ssh_session:
            return {"success": True, "message": "Not connected to SSH server"}
        
        self.ssh_session.close()
        return {"success": True, "message": "Disconnected from SSH server"}


def main():
    """Run the MCP SSH server."""
    # Get custom server name and tool prefix from environment if specified
    server_name = os.environ.get("MCP_SSH_SERVER_NAME")
    tool_prefix = os.environ.get("MCP_SSH_TOOL_PREFIX")
    
    # Create server with config from environment variables
    server = SSHServerMCP(
        server_name=server_name,
        tool_prefix=tool_prefix
    )
    
    # Run the server with stdio transport
    server.run(transport="stdio")


if __name__ == "__main__":
    main()